@@ -387,6 +387,9 @@ dt_get_wcnt_mon(struct dt_d_s that)
 	}
 }
 
+/* forward decl */
+static dt_yd_t dt_conv_to_yd(struct dt_d_s this);
+
 static int
 dt_get_wcnt_year(struct dt_d_s this, unsigned int wkcnt_convention)
 {
@@ -394,24 +397,29 @@ dt_get_wcnt_year(struct dt_d_s this, unsigned int wkcnt_convention)
 
 	switch (this.typ) {
 	case DT_YMD:
+	case DT_DAISY:
+	case DT_YD: {
+		dt_yd_t yd = dt_conv_to_yd(this);
+
 		switch (wkcnt_convention) {
 		default:
 		case YWD_ABSWK_CNT:
-			res = __ymd_get_wcnt_abs(this.ymd);
+			res = __yd_get_wcnt_abs(yd);
 			break;
 		case YWD_ISOWK_CNT:
-			res = __ymd_get_wcnt_iso(this.ymd);
+			res = __yd_get_wcnt_iso(yd);
 			break;
 		case YWD_MONWK_CNT:
 		case YWD_SUNWK_CNT: {
 			/* using monwk_cnt is a minor trick
 			 * from = 1 = Mon or 0 = Sun */
 			int from = wkcnt_convention == YWD_MONWK_CNT;
-			res = __ymd_get_wcnt(this.ymd, from);
+			res = __yd_get_wcnt(yd, from);
 			break;
 		}
 		}
 		break;
+	}
 	case DT_YMCW:
 		res = __ymcw_get_yday(this.ymcw);
 		break;
@@ -636,6 +644,27 @@ dt_conv_to_ywd(struct dt_d_s this)
 	return (dt_ywd_t){.u = 0};
 }
 
+static dt_yd_t
+dt_conv_to_yd(struct dt_d_s this)
+{
+	switch (this.typ) {
+	case DT_YD:
+		/* yay, that was quick */
+		return this.yd;
+	case DT_YMD:
+		return __ymd_to_yd(this.ymd);
+	case DT_DAISY:
+		return __daisy_to_yd(this.daisy);
+	case DT_YMCW:
+		return __ymcw_to_yd(this.ymcw);
+	case DT_YWD:
+		return __ywd_to_yd(this.ywd);
+	default:
+		break;
+	}
+	return (dt_yd_t){.u = 0};
+}
+
 
 /* arithmetic */
 #define ASPECT_ADD
@@ -669,10 +698,19 @@ dt_conv_to_ywd(struct dt_d_s this)
 static const char ymd_dflt[] = "%F";
 static const char ymcw_dflt[] = "%Y-%m-%c-%w";
 static const char ywd_dflt[] = "%rY-W%V-%u";
+static const char yd_dflt[] = "%Y-%d";
 static const char daisy_dflt[] = "%d";
 static const char bizsi_dflt[] = "%db";
 static const char bizda_dflt[] = "%Y-%m-%db";
 
+static const char ymddur_dflt[] = "%Y-%0m-%0d";
+static const char ymcwdur_dflt[] = "%Y-%0m-%0w-%0d";
+static const char ywddur_dflt[] = "%rY-W%0w-%0d";
+static const char yddur_dflt[] = "%Y-%0d";
+static const char daisydur_dflt[] = "%d";
+static const char bizsidur_dflt[] = "%db";
+static const char bizdadur_dflt[] = "%Y-%0m-%0db";
+
 static dt_dtyp_t
 __trans_dfmt_special(const char *fmt)
 {
@@ -711,6 +749,9 @@ __trans_dfmt(const char **fmt)
 		case DT_YWD:
 			*fmt = ywd_dflt;
 			break;
+		case DT_YD:
+			*fmt = yd_dflt;
+			break;
 		case DT_BIZDA:
 			*fmt = bizda_dflt;
 			break;
@@ -725,6 +766,45 @@ __trans_dfmt(const char **fmt)
 	return;
 }
 
+static void
+__trans_ddurfmt(const char **fmt)
+{
+	if (UNLIKELY(*fmt == NULL)) {
+		/* great, standing ovations to the user */
+		;
+	} else if (LIKELY(**fmt == '%')) {
+		/* don't worry about it */
+		;
+	} else {
+		switch (__trans_dfmt_special(*fmt)) {
+		default:
+			break;
+		case DT_YMD:
+			*fmt = ymddur_dflt;
+			break;
+		case DT_YMCW:
+			*fmt = ymcwdur_dflt;
+			break;
+		case DT_YWD:
+			*fmt = ywddur_dflt;
+			break;
+		case DT_YD:
+			*fmt = yddur_dflt;
+			break;
+		case DT_BIZDA:
+			*fmt = bizdadur_dflt;
+			break;
+		case DT_DAISY:
+			*fmt = daisydur_dflt;
+			break;
+		case DT_BIZSI:
+			*fmt = bizsidur_dflt;
+			break;
+		}
+	}
+	return;
+}
+
 /* strpf glue */
 DEFUN struct dt_d_s
 __guess_dtyp(struct strpd_s d)
@@ -1073,16 +1153,31 @@ dt_strfddur(char *restrict buf, size_t bsz, const char *fmt, struct dt_d_s that)
 		d.m = that.ymd.m;
 		d.d = that.ymd.d;
 		if (fmt == NULL) {
-			fmt = ymd_dflt;
+			fmt = ymddur_dflt;
 		}
 		break;
 	case DT_YMCW:
 		d.y = that.ymcw.y;
 		d.m = that.ymcw.m;
 		d.c = that.ymcw.c;
-		d.w = that.ymcw.w;
+		d.d = that.ymcw.w;
 		if (fmt == NULL) {
-			fmt = ymcw_dflt;
+			fmt = ymcwdur_dflt;
+		}
+		break;
+	case DT_YWD:
+		d.y = that.ywd.y;
+		d.c = that.ywd.c;
+		d.d = that.ywd.w;
+		if (fmt == NULL) {
+			fmt = ywddur_dflt;
+		}
+		break;
+	case DT_YD:
+		d.y = that.yd.y;
+		d.d = that.yd.d;
+		if (fmt == NULL) {
+			fmt = yddur_dflt;
 		}
 		break;
 	case DT_DAISY:
@@ -1095,7 +1190,7 @@ dt_strfddur(char *restrict buf, size_t bsz, const char *fmt, struct dt_d_s that)
 		}
 		if (fmt == NULL) {
 			/* subject to change */
-			fmt = daisy_dflt;
+			fmt = daisydur_dflt;
 		}
 		break;
 	case DT_BIZSI:
@@ -1108,7 +1203,7 @@ dt_strfddur(char *restrict buf, size_t bsz, const char *fmt, struct dt_d_s that)
 		}
 		if (fmt == NULL) {
 			/* subject to change */
-			fmt = bizsi_dflt;
+			fmt = bizsidur_dflt;
 		}
 		break;
 	case DT_BIZDA: {
@@ -1123,7 +1218,7 @@ dt_strfddur(char *restrict buf, size_t bsz, const char *fmt, struct dt_d_s that)
 		}
 		d.flags.bizda = 1;
 		if (fmt == NULL) {
-			fmt = bizda_dflt;
+			fmt = bizdadur_dflt;
 		}
 		break;
 	}
@@ -1139,7 +1234,7 @@ dt_strfddur(char *restrict buf, size_t bsz, const char *fmt, struct dt_d_s that)
 		goto out;
 	}
 	/* translate high-level format names */
-	__trans_dfmt(&fmt);
+	__trans_ddurfmt(&fmt);
 
 	/* assign and go */
 	bp = buf;
@@ -1549,7 +1644,18 @@ dt_ddiff(dt_dtyp_t tgttyp, struct dt_d_s d1, struct dt_d_s d2)
 		res = __ymcw_diff(tmp1, tmp2);
 		break;
 	}
-	case DT_YWD:
+	case DT_YD: {
+		dt_yd_t tmp1 = dt_conv_to_yd(d1);
+		dt_yd_t tmp2 = dt_conv_to_yd(d2);
+		res = __yd_diff(tmp1, tmp2);
+		break;
+	}
+	case DT_YWD: {
+		dt_ywd_t tmp1 = dt_conv_to_ywd(d1);
+		dt_ywd_t tmp2 = dt_conv_to_ywd(d2);
+		res = __ywd_diff(tmp1, tmp2);
+		break;
+	}
 	case DT_BIZDA:
 	case DT_DUNK:
 	default:
