@@ -76,29 +76,52 @@ static durfmt_t
 determine_durfmt(const char *fmt)
 {
 	durfmt_t res = {0};
+	dt_dtyp_t special;
 
 	if (fmt == NULL) {
 		/* decide later on */
 		;
-	} else if (strcasecmp(fmt, "ymd") == 0) {
-		res.has_year = 1;
-		res.has_mon = 1;
-		res.has_day = 1;
-	} else if (strcasecmp(fmt, "ymcw") == 0) {
-		res.has_year = 1;
-		res.has_mon = 1;
-		res.has_week = 1;
-		res.has_day = 1;
-	} else if (strcasecmp(fmt, "daisy") == 0) {
-		res.has_day = 1;
-	} else if (strcasecmp(fmt, "bizda") == 0) {
-		res.has_year = 1;
-		res.has_mon = 1;
-		res.has_day = 1;
-		res.has_biz = 1;
-	} else if (strcasecmp(fmt, "bizsi") == 0) {
-		res.has_day = 1;
-		res.has_biz = 1;
+	} else if (UNLIKELY((special = __trans_dfmt_special(fmt)) != DT_DUNK)) {
+		switch (special) {
+		default:
+		case DT_DUNK:
+			break;
+		case DT_YMD:
+			res.has_year = 1;
+			res.has_mon = 1;
+			res.has_day = 1;
+			break;
+		case DT_YMCW:
+			res.has_year = 1;
+			res.has_mon = 1;
+			res.has_week = 1;
+			res.has_day = 1;
+			break;
+		case DT_BIZDA:
+			res.has_year = 1;
+			res.has_mon = 1;
+			res.has_day = 1;
+			res.has_biz = 1;
+			break;
+		case DT_BIZSI:
+			res.has_day = 1;
+			res.has_biz = 1;
+			break;
+		case DT_YWD:
+			res.has_year = 1;
+			res.has_week = 1;
+			res.has_day = 1;
+			break;
+		case DT_YD:
+			res.has_year = 1;
+			res.has_day = 1;
+			break;
+		}
+
+		/* all special types have %0H:%0M:%0S */
+		res.has_hour = 1;
+		res.has_min = 1;
+		res.has_sec = 1;
 	} else {
 		/* go through the fmt specs */
 		for (const char *fp = fmt; *fp;) {
@@ -154,14 +177,6 @@ determine_durfmt(const char *fmt)
 	return res;
 }
 
-static inline bool
-fmt_only_d_p(durfmt_t f)
-{
-	/* from most likely to least likely */
-	return f.flags &&
-		!f.has_sec && !f.has_min && !f.has_hour && !f.has_nano;
-}
-
 static dt_dttyp_t
 determine_durtype(struct dt_dt_s d1, struct dt_dt_s d2, durfmt_t f)
 {
@@ -174,32 +189,32 @@ determine_durtype(struct dt_dt_s d1, struct dt_dt_s d2, durfmt_t f)
 	 *
 	 * where d means a ddur type, t a tdur type and s is DT_SEXY */
 
-	if (UNLIKELY((dt_sandwich_only_d_p(d1) || dt_sandwich_only_d_p(d2)) &&
-		     (dt_sandwich_only_t_p(d1) || dt_sandwich_only_t_p(d2)))) {
+	if (UNLIKELY(dt_sandwich_only_t_p(d1) && dt_sandwich_only_t_p(d2))) {
+		/* time only duration */
 		;
-	} else if ((dt_sandwich_only_d_p(d1) || dt_sandwich_only_d_p(d2)) ||
-		   (dt_sandwich_p(d1) && dt_sandwich_p(d2) &&
-		    fmt_only_d_p(f))) {
-		if (f.has_week && (f.has_mon || f.has_year)) {
-			return (dt_dttyp_t)DT_YMCW;
-		} else if (f.has_mon || f.has_year) {
-			return (dt_dttyp_t)DT_YMD;
-		} else if (f.has_day && f.has_biz) {
-			return (dt_dttyp_t)DT_BIZSI;
-		} else if (f.has_day || f.has_week || f.flags == 0) {
-			return (dt_dttyp_t)DT_DAISY;
-		} else {
-			return (dt_dttyp_t)DT_MD;
-		}
-	} else if ((dt_sandwich_only_t_p(d1) && dt_sandwich_only_t_p(d2)) ||
-		   (dt_sandwich_p(d1) && dt_sandwich_p(d2))) {
-		/* see if we has needs tais */
-		if (UNLIKELY(f.has_tai)) {
-			return (dt_dttyp_t)DT_SEXYTAI;
-		}
-		return (dt_dttyp_t)DT_SEXY;
+	} else if (dt_sandwich_only_t_p(d1) || dt_sandwich_only_t_p(d2)) {
+		/* isn't defined */
+		return (dt_dttyp_t)DT_UNK;
+	} else if (f.has_week && f.has_mon) {
+		return (dt_dttyp_t)DT_YMCW;
+	} else if (f.has_week && f.has_year) {
+		return (dt_dttyp_t)DT_YWD;
+	} else if (f.has_mon) {
+		return (dt_dttyp_t)DT_YMD;
+	} else if (f.has_year && f.has_day) {
+		return (dt_dttyp_t)DT_YD;
+	} else if (f.has_day && f.has_biz) {
+		return (dt_dttyp_t)DT_BIZSI;
+	} else if (f.has_year) {
+		return (dt_dttyp_t)DT_MD;
+	} else if (dt_sandwich_only_d_p(d1) || dt_sandwich_only_d_p(d2)) {
+		/* default date-only type */
+		return (dt_dttyp_t)DT_DAISY;
+	} else if (UNLIKELY(f.has_tai)) {
+		/* we has tais */
+		return (dt_dttyp_t)DT_SEXYTAI;
 	}
-	return DT_UNK;
+	return (dt_dttyp_t)DT_SEXY;
 }
 
 
@@ -222,6 +237,52 @@ error(int eno, const char *fmt, ...)
 	return;
 }
 
+static size_t
+ltostr(char *restrict buf, size_t bsz, long int v,
+       int range, unsigned int pad)
+{
+#define C(x)	(char)((x) + '0')
+	char *restrict bp = buf;
+	const char *const ep = buf + bsz;
+	bool negp;
+
+	if (UNLIKELY((negp = v < 0))) {
+		v = -v;
+	} else if (!v) {
+		*bp++ = C(0U);
+		range--;
+	}
+	/* write the mantissa right to left */
+	for (; v && bp < ep; range--) {
+		register unsigned int x = v % 10U;
+
+		v /= 10U;
+		*bp++ = C(x);
+	}
+	/* fill up with padding */
+	if (UNLIKELY(pad)) {
+		static const char pads[] = " 0";
+		const char p = pads[2U - pad];
+
+		while (range-- > 0) {
+			*bp++ = p;
+		}
+	}
+	/* write the sign */
+	if (UNLIKELY(negp)) {
+		*bp++ = '-';
+	}
+
+	/* reverse the string */
+	for (char *ip = buf, *jp = bp - 1; ip < jp; ip++, jp--) {
+		register char tmp = *ip;
+		*ip = *jp;
+		*jp = tmp;
+	}
+#undef C
+	return bp - buf;
+}
+
 static inline void
 dt_io_warn_dur(const char *d1, const char *d2)
 {
@@ -256,18 +317,6 @@ __strf_tot_corr(struct dt_dt_s dur)
 	return 0;
 }
 
-static long int
-__strf_tot_mins(struct dt_dt_s dur)
-{
-	return __strf_tot_secs(dur) / (long int)SECS_PER_MIN;
-}
-
-static long int
-__strf_tot_hours(struct dt_dt_s dur)
-{
-	return __strf_tot_secs(dur) / (long int)SECS_PER_HOUR;
-}
-
 static int
 __strf_tot_days(struct dt_dt_s dur)
 {
@@ -281,100 +330,28 @@ __strf_tot_days(struct dt_dt_s dur)
 	case DT_BIZSI:
 		d = dur.d.bizsidur;
 		break;
-	default:
-		break;
-	}
-	return d + __strf_tot_secs(dur) / (long int)SECS_PER_DAY;
-}
-
-static int
-__strf_md_days(struct dt_dt_s dur)
-{
-/* DUR expressed as month and days */
-	int d = 0;
-
-	switch (dur.d.typ) {
 	case DT_BIZDA:
 		d = dur.d.bizda.bd;
 		break;
 	case DT_YMD:
 		d = dur.d.ymd.d;
 		break;
+	case DT_YD:
+		d = dur.d.yd.d;
+		break;
 	case DT_YMCW:
-		d = dur.d.ymcw.w + dur.d.ymcw.c * (int)GREG_DAYS_P_WEEK;
+		d = dur.d.ymcw.w + dur.d.ymcw.c * (int)GREG_DAYS_P_WEEK;;
+		break;
+	case DT_YWD:
+		d = dur.d.ywd.w + dur.d.ywd.c * (int)GREG_DAYS_P_WEEK;
+		break;
+	case DT_MD:
+		d = dur.d.md.d;
 		break;
 	default:
 		break;
 	}
-	return d + __strf_tot_secs(dur) / (long int)SECS_PER_DAY;
-}
-
-static int
-__strf_yd_days(struct dt_dt_s dur)
-{
-/* DUR expressed as years and days */
-	int d = 0;
-
-	switch (dur.d.typ) {
-	case DT_BIZDA:
-		d = dur.d.bizda.bd;
-		break;
-	case DT_YMD:
-		d = dur.d.ymd.d;
-		break;
-	case DT_YMCW:
-		d = dur.d.ymcw.w + dur.d.ymcw.c * (int)GREG_DAYS_P_WEEK;
-		break;
-	default:
-		break;
-	}
-	return d + __strf_tot_secs(dur) / (long int)SECS_PER_DAY;
-}
-
-static int
-__strf_w_days(struct dt_dt_s dur)
-{
-/* DUR expressed as something, weeks and days */
-	int d = 0;
-
-	switch (dur.d.typ) {
-	case DT_DAISY:
-		d = dur.d.daisydur % (int)GREG_DAYS_P_WEEK;
-		break;
-	case DT_BIZSI:
-		d = dur.d.bizsidur % (int)GREG_DAYS_P_WEEK;
-		break;
-	case DT_BIZDA:
-		d = dur.d.bizda.bd % (int)GREG_DAYS_P_WEEK;
-		break;
-	case DT_YMD:
-		d = dur.d.ymd.d % (int)GREG_DAYS_P_WEEK;
-		break;
-	case DT_YMCW:
-		d = dur.d.ymcw.w;
-		break;
-	default:
-		break;
-	}
-	return d + __strf_tot_secs(dur) / (long int)SECS_PER_DAY;
-}
-
-static int
-__strf_tot_weeks(struct dt_dt_s dur)
-{
-	return __strf_tot_days(dur) / (int)GREG_DAYS_P_WEEK;
-}
-
-static int
-__strf_md_weeks(struct dt_dt_s dur)
-{
-	return __strf_md_days(dur) / (int)GREG_DAYS_P_WEEK;
-}
-
-static int
-__strf_yd_weeks(struct dt_dt_s dur)
-{
-	return __strf_yd_days(dur) / (int)GREG_DAYS_P_WEEK;
+	return d;
 }
 
 static int
@@ -396,6 +373,12 @@ __strf_tot_mon(struct dt_dt_s dur)
 	case DT_YMCW:
 		m = dur.d.ymcw.m + dur.d.ymcw.y * (int)GREG_MONTHS_P_YEAR;
 		break;
+	case DT_YD:
+		m = dur.d.yd.y * (int)GREG_MONTHS_P_YEAR;
+		break;
+	case DT_YWD:
+		m = dur.d.ywd.y * (int)GREG_MONTHS_P_YEAR;
+		break;
 	default:
 		break;
 	}
@@ -414,14 +397,92 @@ __strf_tot_years(struct dt_dt_s dur)
 	return __strf_tot_mon(dur) / (int)GREG_MONTHS_P_YEAR;
 }
 
+static struct precalc_s {
+	int Y;
+	int m;
+	int w;
+	int d;
+	int db;
+
+	long int H;
+	long int M;
+	long int S;
+	long int N;
+
+	long int rS;
+} precalc(durfmt_t f, struct dt_dt_s dur)
+{
+#define MINS_PER_DAY	(MINS_PER_HOUR * HOURS_PER_DAY)
+#define SECS_PER_WEEK	(SECS_PER_DAY * GREG_DAYS_P_WEEK)
+#define MINS_PER_WEEK	(MINS_PER_DAY * GREG_DAYS_P_WEEK)
+#define HOURS_PER_WEEK	(HOURS_PER_DAY * GREG_DAYS_P_WEEK)
+	struct precalc_s res = {0};
+
+	/* date specs */
+	if (f.has_year) {
+		/* just years */
+		res.Y = __strf_tot_years(dur);
+	}
+	if (f.has_year && f.has_mon) {
+		/* years and months */
+		res.m = __strf_ym_mon(dur);
+	} else if (f.has_mon) {
+		/* just months */
+		res.m = __strf_tot_mon(dur);
+	}
+	if (f.has_day || f.has_week) {
+		res.d = __strf_tot_days(dur);
+		if (f.has_week) {
+			/* week shadows days in the hierarchy */
+			res.w = res.d / (int)GREG_DAYS_P_WEEK;
+			res.d %= (int)GREG_DAYS_P_WEEK;
+		}
+	}
+
+	/* time specs */
+	if (f.has_sec || f.has_min || f.has_hour) {
+		res.S = __strf_tot_secs(dur);
+
+		/* carry from the date specs */
+		res.S += res.w * (long int)SECS_PER_WEEK;
+		res.S += res.d * (long int)SECS_PER_DAY;
+
+		if (f.has_week) {
+			res.w = res.S / (long int)SECS_PER_WEEK;
+			if ((res.S %= (long int)SECS_PER_WEEK) < 0) {
+				res.S += SECS_PER_WEEK;
+				res.w--;
+			}
+		}
+		if (f.has_day) {
+			res.d = res.S / (long int)SECS_PER_DAY;
+			if ((res.S %= (long int)SECS_PER_DAY) < 0) {
+				res.S += SECS_PER_DAY;
+				res.d--;
+			}
+		}
+		if (f.has_hour) {
+			res.H = res.S / (long int)SECS_PER_HOUR;
+			res.S %= (long int)SECS_PER_HOUR;
+		}
+		if (f.has_min) {
+			/* minutes and seconds */
+			res.M = res.S / (long int)SECS_PER_MIN;
+			res.S %= (long int)SECS_PER_MIN;
+		}
+	}
+	return res;
+}
+
 static size_t
 __strfdtdur(
 	char *restrict buf, size_t bsz, const char *fmt,
 	struct dt_dt_s dur, durfmt_t f)
 {
 /* like strfdtdur() but do some calculations based on F on the way there */
-	static const char sexy_dflt_dur[] = "%T";
+	static const char sexy_dflt_dur[] = "%0T";
 	static const char ddur_dflt_dur[] = "%d";
+	struct precalc_s pre;
 	const char *fp;
 	char *bp;
 
@@ -434,32 +495,41 @@ __strfdtdur(
 	if (dur.typ == DT_SEXY) {
 		if (fmt == NULL) {
 			fmt = sexy_dflt_dur;
+			f.has_sec = 1U;
 		}
 	} else if (dur.typ == DT_SEXYTAI) {
 		if (fmt == NULL) {
 			fmt = sexy_dflt_dur;
+			f.has_sec = 1U;
 		}
 	} else if (dt_sandwich_p(dur)) {
 		if (fmt == NULL) {
 			fmt = sexy_dflt_dur;
+			f.has_sec = 1U;
 		} else {
-			__trans_dtfmt(&fmt);
+			__trans_dtdurfmt(&fmt);
 		}
 	} else if (dt_sandwich_only_d_p(dur)) {
 		if (fmt == NULL) {
 			fmt = ddur_dflt_dur;
+			f.has_day = 1U;
 		} else {
-			__trans_dfmt(&fmt);
+			__trans_ddurfmt(&fmt);
+			f.has_day = 1U;
 		}
 	} else if (dt_sandwich_only_t_p(dur)) {
 		if (fmt == NULL) {
 			fmt = sexy_dflt_dur;
+			f.has_sec = 1U;
 		}
 	} else {
 		bp = buf;
 		goto out;
 	}
 
+	/* precompute */
+	pre = precalc(f, dur);
+
 	/* assign and go */
 	bp = buf;
 	fp = fmt;
@@ -491,28 +561,18 @@ __strfdtdur(
 			*bp++ = '\n';
 			break;
 
-		case DT_SPFL_N_DSTD: {
-			int d = __strf_tot_days(dur);
-			bp += snprintf(bp, eo - bp, "%dd", d);
+		case DT_SPFL_N_DSTD:
+			bp += ltostr(bp, eo - bp, pre.d, -1, DT_SPPAD_NONE);
+			*bp++ = 'd';
 			goto bizda_suffix;
-		}
-		case DT_SPFL_N_DCNT_MON: {
-			int d;
 
-			if (f.has_week) {
-				/* week shadows days in the hierarchy */
-				d = __strf_w_days(dur);
-			} else if (!f.has_week && f.has_mon) {
-				/* days and months */
-				d = __strf_md_days(dur);
-			} else if (!f.has_mon && f.has_year) {
-				/* days and years */
-				d = __strf_yd_days(dur);
-			} else {
-				/* just days */
-				d = __strf_tot_days(dur);
+		case DT_SPFL_N_DCNT_MON: {
+			int rng = 2;
+
+			if (!f.has_mon && !f.has_week && f.has_year) {
+				rng++;
 			}
-			bp += snprintf(bp, eo - bp, "%d", d);
+			bp += ltostr(bp, eo - bp, pre.d, rng, spec.pad);
 		}
 		bizda_suffix:
 			if (spec.bizda) {
@@ -527,95 +587,44 @@ __strfdtdur(
 				}
 			}
 			break;
+
 		case DT_SPFL_N_WCNT_MON:
-		case DT_SPFL_N_DCNT_WEEK: {
-			int w;
-
-			if (f.has_mon) {
-				/* months and weeks */
-				w = __strf_md_weeks(dur);
-			} else if (f.has_year) {
-				/* years and weeks */
-				w = __strf_yd_weeks(dur);
-			} else {
-				/* just weeks */
-				w = __strf_tot_weeks(dur);
-			}
-			bp += snprintf(bp, eo - bp, "%d", w);
+		case DT_SPFL_N_DCNT_WEEK:
+			bp += ltostr(bp, eo - bp, pre.w, 2, spec.pad);
 			break;
-		}
-		case DT_SPFL_N_MON: {
-			int m;
 
-			if (f.has_year) {
-				/* years and months */
-				m = __strf_ym_mon(dur);
-			} else {
-				/* just months */
-				m = __strf_tot_mon(dur);
-			}
-			bp += snprintf(bp, eo - bp, "%d", m);
+		case DT_SPFL_N_MON:
+			bp += ltostr(bp, eo - bp, pre.m, 2, spec.pad);
 			break;
-		}
-		case DT_SPFL_N_YEAR: {
-			int y;
 
-			/* just years */
-			y = __strf_tot_years(dur);
-			bp += snprintf(bp, eo - bp, "%d", y);
+		case DT_SPFL_N_YEAR:
+			bp += ltostr(bp, eo - bp, pre.Y, -1, DT_SPPAD_NONE);
 			break;
-		}
 
 			/* time specs */
-		case DT_SPFL_N_TSTD: {
-			long int s = __strf_tot_secs(dur);
-
+		case DT_SPFL_N_TSTD:
 			if (UNLIKELY(spec.tai)) {
-				s += __strf_tot_corr(dur);
+				pre.S += __strf_tot_corr(dur);
 			}
-			bp += snprintf(bp, eo - bp, "%lds", s);
+			bp += ltostr(bp, eo - bp, pre.S, -1, DT_SPPAD_NONE);
+			*bp++ = 's';
 			break;
-		}
-		case DT_SPFL_N_SEC: {
-			long int s = __strf_tot_secs(dur);
 
-			if (f.has_min) {
-				/* minutes and seconds */
-				s %= (long int)SECS_PER_MIN;
-			} else if (f.has_hour) {
-				/* hours and seconds */
-				s %= (long int)SECS_PER_HOUR;
-			} else if (f.has_day) {
-				s %= (long int)SECS_PER_DAY;
-			}
+		case DT_SPFL_N_SEC:
 			if (UNLIKELY(spec.tai)) {
-				s += __strf_tot_corr(dur);
+				pre.S += __strf_tot_corr(dur);
 			}
-			bp += snprintf(bp, eo - bp, "%ld", s);
-			break;
-		}
-		case DT_SPFL_N_MIN: {
-			long int m = __strf_tot_mins(dur);
 
-			if (f.has_hour) {
-				/* hours and minutes */
-				m %= (long int)MINS_PER_HOUR;
-			} else if (f.has_day) {
-				m %= (long int)(MINS_PER_HOUR * HOURS_PER_DAY);
-			}
-			bp += snprintf(bp, eo - bp, "%ld", m);
+			bp += ltostr(bp, eo - bp, pre.S, 2, spec.pad);
 			break;
-		}
-		case DT_SPFL_N_HOUR: {
-			long int h = __strf_tot_hours(dur);
 
-			if (f.has_day) {
-				/* hours and days */
-				h %= (long int)HOURS_PER_DAY;
-			}
-			bp += snprintf(bp, eo - bp, "%ld", h);
+		case DT_SPFL_N_MIN:
+			bp += ltostr(bp, eo - bp, pre.M, 2, spec.pad);
+			break;
+
+		case DT_SPFL_N_HOUR:
+			bp += ltostr(bp, eo - bp, pre.H, 2, spec.pad);
 			break;
-		}
 
 		default:
 			break;
@@ -653,8 +662,8 @@ ddiff_prnt(struct dt_dt_s dur, const char *fmt, durfmt_t f)
 # pragma GCC diagnostic ignored "-Wswitch-enum"
 # pragma GCC diagnostic ignored "-Wunused-function"
 #endif	/* __INTEL_COMPILER */
-#include "ddiff-clo.h"
-#include "ddiff-clo.c"
+#include "ddiff.xh"
+#include "ddiff.x"
 #if defined __INTEL_COMPILER
 # pragma warning (default:593)
 # pragma warning (default:181)
